/*
 * Samsung API
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "main_app.h"

#include "view/layout_view.h"
#include "utils/logger.h"
#include "model/constants.h"
#include <stdlib.h>
#include "model/notification_control.h"
#include <notification.h>
#include <Elementary.h>
#include <app.h>
#include "utils/config.h"
#include <efl_extension.h>
#include <badge.h>


// app event callbacks
static bool _on_create_cb(void *user_data);
static void _on_terminate_cb(void *user_data);
static void _on_pause_cb(void *user_data);
static void _on_resume_cb(void *user_data);
static void _on_app_control_cb(app_control_h app_control, void *user_data);
static void _app_navi_back_cb(void *data, Evas_Object *obj, void *event_info);

app_data *app_create()
{
	app_data *app = calloc(1, sizeof(app_data));
	return app;
}

void app_destroy(app_data *app)
{
	if (app) {
		free(app);
	}
}

int app_run(app_data *app, int argc, char **argv)
{
	if (!app) {
		return -1;
	}

	ui_app_lifecycle_callback_s cbs =
	{
		.create = _on_create_cb,
		.terminate = _on_terminate_cb,
		.pause = _on_pause_cb,
		.resume = _on_resume_cb,
		.app_control = _on_app_control_cb
	};

	return ui_app_main(argc, argv, &cbs, app);
}

static Evas_Object *_add_naviframe(app_data *ad)
{
	Evas_Object *navi = elm_naviframe_add((win_get_host_layout(ad->win)));
	retvm_if(!navi, NULL, "navi is NULL");

	elm_naviframe_prev_btn_auto_pushed_set(navi, EINA_FALSE);
	eext_object_event_callback_add(navi, EEXT_CALLBACK_BACK, _app_navi_back_cb, ad);
	win_set_layout(ad->win, navi);

	return navi;
}

static bool _on_create_cb(void *user_data)
{
	Evas_Object *box;

	app_data *ad = user_data;
	retvm_if(!ad, false, "ad is NULL");

	ad->win = win_create();
	retvm_if(!ad->win, false, "ad->win is NULL");

	ad->navi = _add_naviframe(ad);
	retvm_if(!ad->navi, false, "ad->navi is NULL");

	box = layout_view_add_scroll_list(ad->navi, MAIN_LIST);
	retvm_if(!box, false, "box is NULL");

	/* Create badge */
	int err = badge_new(PACKAGE);
	if (BADGE_ERROR_NONE != err) {
		_E("Badge is not created.");
	}

	elm_naviframe_item_push(ad->navi, title_main , NULL, NULL, box, NULL);

	return true;
}

static void _app_navi_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	retm_if(!data, "data is NULL");
	app_data *app = data;

	if (elm_naviframe_top_item_get(app->navi) == elm_naviframe_bottom_item_get(app->navi)) 	{
		win_lower(app->win);
	} else {
		elm_naviframe_item_pop(app->navi);
	}
}

static void _on_terminate_cb(void *user_data)
{
	app_data *ad = user_data;

	if (!ad) {
		return;
	}

	win_destroy(ad->win);
	ad->win = NULL;
	ad->navi= NULL;
}

static void _on_pause_cb(void *user_data)
{

}

static void _on_resume_cb(void *user_data)
{

}

static void _on_app_control_cb(app_control_h app_control, void *user_data)
{
	retm_if(!user_data, "user_data is NULL");

	app_data *app = (app_data *)user_data;
	char *argument= NULL;
	char *uri_data = NULL;
	char *operation_data = NULL;
	char *mime_data = NULL;
	char **result_service_data = NULL;
	int length = 0;
	char extra_data_text[TEXT_SHORT_LEN] = {'\0'};
	char **result_service_data_to = NULL;
	char extra_data_to[TEXT_SHORT_LEN] = { '\0'};
	int err = 0;

	app_control_get_extra_data(app_control, MESSAGE_POST, &argument);
	retm_if(!argument, "argument is NULL");

	app_control_get_uri(app_control, &uri_data);

	if (!uri_data) {
		free(argument);
		return;
	}

	notification_data* notify_data = (notification_data *) malloc(sizeof(notification_data));

	if (!notify_data) {
		free(argument);
		free(uri_data);
		return;
	}

	snprintf(notify_data->name, TEXT_SHORT_LEN, "%s", app_control_request);
	app_control_get_operation(app_control, &operation_data);
	app_control_get_mime(app_control, &mime_data);

	err = app_control_get_extra_data_array(app_control, SERVICE_DATA_TEXT, &result_service_data, &length);
	if (err == APP_CONTROL_ERROR_NONE) {
		int i = 0;
		snprintf(extra_data_text, TEXT_SHORT_LEN, "# 1%s (%s | %s)<br>", key_value, SERVICE_DATA_TEXT, result_service_data[0]);
		for (; i != length; i++) {
			free(result_service_data[i]);
		}
		free(result_service_data);
	}

	err = app_control_get_extra_data_array(app_control, SERVICE_DATA_TO, &result_service_data_to, &length);
	if (err == APP_CONTROL_ERROR_NONE) {
		int i = 0;
		snprintf(extra_data_to, TEXT_SHORT_LEN, "# 2%s (%s | %s)", key_value, SERVICE_DATA_TEXT, result_service_data_to[0]);
		for (; i != length; i++) {
			free(result_service_data_to[i]);
		}
		free(result_service_data_to);
	}

	snprintf(notify_data->result_text, TEXT_MAX_LEN, "%s %i<br>%s %s<br>%s %s<br>%s %s<br><br>%s<br># 0%s (%s | %s)<br>%s%s",
			req_id, ++app->service_request_count,operation_id,operation_data,uri_data_text,uri_data
			,mime_type, mime_data,extra_data, key_value, extra_data_tizen_org, argument, extra_data_text, extra_data_to);

	if ( layout_view_result_view_add(app->navi, notify_data) == NULL) {
		_E("add reult view fail");
	}

	free(argument);
	free(uri_data);
	free(mime_data);
	free(notify_data);
	free(operation_data);
	win_raise(app->win);
}

