/*
 * Samsung API
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "view/layout_view.h"
#include "utils/config.h"
#include "utils/logger.h"
#include <Elementary.h>
#include "model/notification_control.h"
#include "model/constants.h"

Evas_Object *layout_view_result_view_add(Evas_Object *navi, notification_data *notify_info)
{
	Evas_Object *scroller;
	Evas_Object *box;
	Evas_Object *content;

	retvm_if(!navi, NULL, "navi is NULL");

	scroller = elm_scroller_add(navi);
	retvm_if(!scroller, NULL, "scroller is NULL");

	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
	evas_object_show (scroller);

	box = elm_box_add(scroller);
	retvm_if(!box, NULL, "box is NULL");
	
	elm_object_content_set(scroller, box);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_horizontal_set(box, EINA_FALSE);
	evas_object_show(box);

	content = elm_label_add(box);
	retvm_if(!content, NULL, "content is NULL");
	
	elm_label_line_wrap_set(content, ELM_WRAP_MIXED);
	elm_object_text_set(content, notify_info->result_text);
	evas_object_size_hint_weight_set(content, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(content, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_pack_start(box, content);
	evas_object_show(content);

	elm_naviframe_item_push(navi, result_title, NULL, NULL , scroller , NULL);

	return scroller;

}


static void _add_noti_list(Evas_Object *navi, int list_type)
{
	Evas_Object *scroller;

	scroller = layout_view_add_scroll_list(navi,list_type);
	retm_if(!scroller, "scroller is NULL");

	if (list_type == NOTIFICATION_LIST) {
		elm_naviframe_item_push(navi, title_notification , NULL, NULL, scroller, NULL);
	} else {
		elm_naviframe_item_push(navi, title_ongoing_notification , NULL, NULL, scroller, NULL);
	}
}


static void _noti_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	Evas_Object *navi = (Evas_Object *)data;
	retm_if(!navi,"navi is NULL");

	_add_noti_list(navi , NOTIFICATION_LIST);
}

static void _ongoing_noti_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	Evas_Object *navi = (Evas_Object *)data;
	retm_if(!navi, "navi is NULL");

	_add_noti_list(navi , ONGOING_NOTIFICATION_LIST);
}

static void _noti_list_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	cb_data *cd = (cb_data*) data;
	retm_if(!cd, "cd is NULL");

	notification_data *notification_info = (notification_data *)cd->notify_data;
	notification_info->callback(notification_info);

	if ( layout_view_result_view_add(cd->navi, notification_info) == NULL) {
		_E(" layout_view_result_view_add is failed");
	}
}

static void _destoy_layout_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	cb_data *cd = (cb_data *)data;
	if (cd)	{
		free(cd);
	}
}

static Evas_Object *_add_textblock_layout(Evas_Object* parent, Edje_Signal_Cb cb, void *cb_data, const char * label)
{
	Evas_Object *layout;

	layout = elm_layout_add(parent);
	retvm_if(!layout, NULL, "layout is NULL");

	elm_layout_file_set(layout, LISTEDJ, "list/content/text");
	evas_object_size_hint_weight_set (layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_text_set(layout, "elm.text", label);
	elm_object_signal_callback_add( layout, "clicked", "*",  cb , cb_data);
	evas_object_show(layout);

	return layout;
}

Evas_Object *layout_view_add_scroll_list(Evas_Object *navi, int list_type)
{
	Evas_Object *scroller;
	Evas_Object *box;
	Evas_Object *layout;

	scroller = elm_scroller_add(navi);
	retvm_if(!scroller, NULL, "scroller is NULL");

	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF , ELM_SCROLLER_POLICY_OFF);
	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_fill_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(scroller);

	box = elm_box_add(scroller);
	retvm_if(!box, NULL, "box is NULL");

	evas_object_size_hint_weight_set(box, 0.0, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_content_set(scroller, box);
	evas_object_show(box);

	if (list_type == MAIN_LIST) {
		layout = _add_textblock_layout(box, _noti_cb, navi, title_notification);
		retvm_if(!layout, NULL, "layout is NULL");
		elm_box_pack_end(box, layout);

		layout = _add_textblock_layout(box, _ongoing_noti_cb, navi, title_ongoing_notification);
		retvm_if(!layout, NULL, "layout is NULL");
		elm_box_pack_end(box, layout);
	} else {
		int index = 0;
		int size = 0;
		notification_data *notification_list;

		if (list_type == NOTIFICATION_LIST) {
			notification_list = notification_contol_list_get(&size);
		} else {//ONGOING_NOTIFICATION_LIST
			notification_list = notification_control_ongoing_list_get(&size);
		}

		for (index = 0; index < size; index++) {
			cb_data *cd = calloc(1, sizeof(cb_data));
			retvm_if(!cd , NULL, "cd  is NULL");

			cd->navi = navi;
			cd->notify_data = &notification_list[index];

			layout = _add_textblock_layout(box, _noti_list_cb, cd, notification_list[index].name);
			retvm_if(!layout, NULL, "layout is NULL");

			evas_object_event_callback_add(layout, EVAS_CALLBACK_FREE, _destoy_layout_cb, cd);
			elm_box_pack_end(box, layout);
		}
	}

	return scroller;
}


